/**
 *  OdetteJ - Open Source Odette FTP implementation in Java(TM).
 *  Copyright (C) 2005 Fossil E-Commerce, http://www.fossilec.com
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  ------------------------------------------------------------------------
 *  $Id: RecordFormat.java,v 1.2 2005/08/15 16:43:37 rmarins Exp $
 *  ------------------------------------------------------------------------
 */

package org.fossilec.odettej.service;

import java.util.Collection;
import java.util.Map;
import java.util.WeakHashMap;

public final class RecordFormat {

	/**
	 * Each record in the file has the same length.
	 */
	public static final RecordFormat FIXED = new RecordFormat("F");

	/**
	 * The records in the file can have different lengths.
	 */
	public static final RecordFormat VARIABLE = new RecordFormat("V");

	/**
	 * The file contains a stream of data. No structure is defined.
	 */
	public static final RecordFormat UNSTRUCTURED = new RecordFormat("U");

	/**
	 * A Text File is defined as a sequence of ASCII characters, containing no
	 * control characters except CR/LF which delimits lines. A line will not
	 * have more than 2048 characters.
	 */
	public static final RecordFormat TEXTFILE = new RecordFormat("T");

	private static final Map ENUM_MAP = new WeakHashMap(14);

	static {
		addEnum(FIXED);
		addEnum(VARIABLE);
		addEnum(UNSTRUCTURED);
		addEnum(TEXTFILE);
	}

	private String code;

	/**
	 * Enumeration private constructor with record format code.
	 * 
	 * @param aCode
	 *            Unique class instance identifier.
	 */
	private RecordFormat(String formatCode) {
		super();
		this.code = formatCode;
	}

	public String getCode() {
		return this.code;
	}

	public String toString() {
		return getCode();
	}

	/**
	 * Convenient method for parsing the proper CommandIdentifier instance given
	 * a identifier character.
	 * 
	 * @param identifier
	 *            The command identifier being evaluated
	 * @return CommandIdentifier Instance that correspond to the parameter
	 * @throws CommandNotRecognisedException
	 *             Command not recognised
	 */
	public static RecordFormat parse(String code) {
		return (RecordFormat) EnumUtil.parseEnum(ENUM_MAP, code);
	}

	private static void addEnum(RecordFormat type) {
		ENUM_MAP.put(type.getCode(), type);
	}

	public static Collection values() {
		return ENUM_MAP.values();
	}
}
