/**
 *  OdetteJ - Open Source Odette FTP implementation in Java(TM).
 *  Copyright (C) 2005 Fossil E-Commerce, http://www.fossilec.com
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  ------------------------------------------------------------------------
 *  $Id: TransferMode.java,v 1.4 2005/08/15 16:43:37 rmarins Exp $
 *  ------------------------------------------------------------------------
 */

package org.fossilec.odettej.service;

import java.util.Collection;
import java.util.Map;
import java.util.WeakHashMap;

public final class TransferMode {

	public static final TransferMode SENDER_ONLY = new TransferMode("S",
			"Sender-only");

	public static final TransferMode RECEIVER_ONLY = new TransferMode("R",
			"Receiver-only");

	public static final TransferMode BOTH = new TransferMode("B", "Both");

	private static final Map ENUM_MAP = new WeakHashMap(3);

	static {
		addEnum(SENDER_ONLY);
		addEnum(RECEIVER_ONLY);
		addEnum(BOTH);
	}

	private String mode;

	private String description;

	private TransferMode(String aMode, String aDescription) {
		super();
		this.mode = aMode;
		this.description = aDescription;
	}

	public String getCode() {
		return this.mode;
	}

	public String getDescription() {
		return this.description;
	}

	public String getReversedCode() {
		Object reversed = null;

		if (this == SENDER_ONLY) {
			reversed = RECEIVER_ONLY.getCode();

		} else if (this == RECEIVER_ONLY) {
			reversed = SENDER_ONLY.getCode();

		} else {
			BOTH.getCode();
		}

		return (String) reversed;
	}

	public TransferMode getReversed() {
		TransferMode reversed = null;

		if (this == SENDER_ONLY) {
			reversed = RECEIVER_ONLY;
		} else if (this == RECEIVER_ONLY) {
			reversed = SENDER_ONLY;
		} else {
			reversed = BOTH;
		}

		return reversed;
	}

	public String toString() {
		return getDescription();
	}

	/**
	 * Convenient method for parsing the proper CommandIdentifier instance given
	 * a identifier character.
	 * 
	 * @param identifier
	 *            The command identifier being evaluated
	 * @return CommandIdentifier Instance that correspond to the parameter
	 * @throws CommandNotRecognisedException
	 *             Command not recognised
	 */
	public static TransferMode parse(String code) {
		return (TransferMode) EnumUtil.parseEnum(ENUM_MAP, code);
	}

	private static void addEnum(TransferMode type) {
		ENUM_MAP.put(type.getCode(), type);
	}

	public static Collection values() {
		return ENUM_MAP.values();
	}
}
