/**
 *  OdetteJ - Open Source Odette FTP implementation in Java(TM).
 *  Copyright (C) 2005 Fossil E-Commerce, http://www.fossilec.com
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  ------------------------------------------------------------------------
 *  $Id: UsualService.java,v 1.5 2005/09/11 14:23:57 rmarins Exp $
 *  ------------------------------------------------------------------------
 */

package org.fossilec.odettej.service;

import java.nio.ByteBuffer;

import org.fossilec.odettej.OdetteFTPException;
import org.fossilec.odettej.transport.NotConnectedException;
import org.fossilec.odettej.transport.Transport;

/**
 * 
 * @author Rafael Marins
 */
public class UsualService extends Service {

	public UsualService(Transport transport, int debSize) {
		super(transport, debSize);
	}

	public OdetteExchangeBuffer receive(boolean strictFormat)
			throws OdetteFTPException {

		if (!isConnected()) {
			throw new NotConnectedException();
		}

		/* receive and adjust limit to the received data */
		ByteBuffer buffer = receiveOdetteExchangeBuffer();

		OdetteExchangeBuffer oeb = OdetteExchangeBuffer.getInstance(buffer,
				strictFormat);

		return oeb;
	}

	private ByteBuffer receiveOdetteExchangeBuffer() throws OdetteFTPException {

		Transport transport = getTransport();

		/* receive command identifier buffer */
		ByteBuffer ib = ByteBuffer.allocateDirect(1);
		transport.receive(ib);

		ib.rewind();
		CommandIdentifier ci = OdetteExchangeBuffer.getIdentifier(ib);

		/*
		 * The block below identify the receiving buffer to allocate the correct
		 * buffer size for receiving data. In case of data packets the maximum
		 * exchange buffer will be used.
		 */

		ByteBuffer buffer;

		if (ci == CommandIdentifier.DATA) {
			/* call method to receive a Data Exchange Buffer */
			buffer = receiveDataExchangeBuffer();

		} else if ((ci == CommandIdentifier.CD)) {
			/* short commands are enough */
			ib.rewind();
			buffer = ib;

		} else {
			/* call method to receive the complete Command Exchange Buffer */
			buffer = receiveCommandExchangeBuffer(ci);
		}

		return buffer;
	}

	private ByteBuffer receiveDataExchangeBuffer() throws OdetteFTPException {

		Transport transport = getTransport();

		/*
		 * Allocate the data exchange buffer in hope for receiving the the
		 * complete buffer and exit.
		 */

		ByteBuffer buffer = ByteBuffer.allocateDirect(getExchangeBufferSize());

		/* insert DATA command identifier already read from channel */
		String code = CommandIdentifier.DATA.getCode();
		buffer.put(OdetteExchangeBuffer.unwrapString(code));

		transport.receive(buffer);

		buffer.rewind();
		return buffer;
	}

	private ByteBuffer receiveCommandExchangeBuffer(CommandIdentifier ci)
			throws OdetteFTPException {

		Transport transport = getTransport();

		/*
		 * Discover the command exchange buffer size by its format and allocate
		 * the correct length to receive remaining buffer.
		 */

		CommandFormat cf = CommandFormat.getFormat(ci);
		ByteBuffer buffer = ByteBuffer.allocateDirect(cf.getLength());

		buffer.put(OdetteExchangeBuffer.unwrapString(ci.getCode()));

		if (buffer.hasRemaining()) {
			transport.receive(buffer);
		}

		buffer.rewind();
		return buffer;
	}

	public int send(OdetteExchangeBuffer oeb) throws OdetteFTPException {
		Transport transport = getTransport();
		int octetsSent = transport.send(oeb.getBuffer());

		return octetsSent;
	}
}
